# BEGIN COPYRIGHT BLOCK
# This Program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 2 of the License.
# 
# This Program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA.
# 
# Copyright (C) 2007 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#

package AdminServer;
require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(createAdminServer reconfigAdminServer
                createASFilesAndDirs setFileOwnerPerms updateHttpConfFiles
                startAdminServer);
@EXPORT_OK = qw(createAdminServer reconfigAdminServer
                createASFilesAndDirs setFileOwnerPerms updateHttpConfFiles
                startAdminServer);

use File::Path;
# tempfiles
use File::Temp qw(tempfile tempdir);

# load perldap
use Mozilla::LDAP::Conn;
use Mozilla::LDAP::Utils qw(normalizeDN);
use Mozilla::LDAP::API qw(ldap_url_parse);
use Mozilla::LDAP::LDIF;

use Util;
use Inf;
use Setup;
use AdminUtil;

sub checkRequiredParameters {
    my $setup = shift;
    for my $asparam (qw(Port SysUser ServerAdminID ServerAdminPwd)) {
        if (!defined($setup->{inf}->{admin}->{$asparam})) {
            $setup->msg($FATAL, "missing_adminserver_param", $asparam);
            return 0;
        }
    }

    for my $general (qw(SuiteSpotUserID SuiteSpotGroup ConfigDirectoryLdapURL)) {
        if (!defined($setup->{inf}->{General}->{$general})) {
            $setup->msg($FATAL, "missing_general_param", $general);
            return 0;
        }
    }

    return 1;
}

sub setFileOwnerPerms {
    my $setup = shift;
    my $configdir = shift;
    my $admConf = getAdmConf($configdir);
    my $uid = getpwnam $admConf->{sysuser};

    # chown and chmod other files appropriately
    for (glob("$configdir/*")) {
        # these are owned by root
        next if (/httpd.conf$/);
        next if (/nss.conf$/);
        next if (/admserv.conf$/);
        next if (! -f $_); # should never happen
        # all other files should be owned by SysUser
        $! = 0; # clear errno
        chown $uid, -1, $_;
        if ($!) {
            $setup->msg($FATAL, 'error_chowning_file', $_,
                        $admConf->{sysuser}, $!);
            return 0;
        }
        # the files should be writable
        $! = 0; # clear errno
        chmod 0600, $_;
        if ($!) {
            $setup->msg($FATAL, 'error_chmoding_file', $_, $!);
            return 0;
        }
    }

    return 1;
}

sub createASFilesAndDirs {
    my $setup = shift;
    my $configdir = shift;
    my $securitydir = shift;
    my $logdir = shift;
    my $rundir = shift;

    my $uid = getpwnam $setup->{inf}->{admin}->{SysUser};
    my $gid = getgrnam $setup->{inf}->{General}->{SuiteSpotGroup};

    $setup->msg('create_adminserver_filesdirs');

    # these paths are owned exclusively by admin sever
    my @errs;
    for ($configdir, $securitydir, $logdir) {
        @errs = makePaths($_, 0700, $setup->{inf}->{admin}->{SysUser},
                          $setup->{inf}->{General}->{SuiteSpotGroup});
        if (@errs) {
            $setup->msg($FATAL, @errs);
            return 0;
        }
        $! = 0; # clear errno
        chmod 0700, $_;
        if ($!) {
            $setup->msg($FATAL, 'error_chmoding_directory', $_, $!);
            return 0;
        }
        chown $uid, -1, $_;
        if ($!) {
            $setup->msg($FATAL, 'error_chowning_directory', $_,
                        $setup->{inf}->{admin}->{SysUser}, $!);
            return 0;
        }
    }

    # these paths are shared by SuiteSpotGroup members
    @errs = makePaths($rundir, 0770, $setup->{inf}->{admin}->{SysUser},
                      $setup->{inf}->{General}->{SuiteSpotGroup});
    if (@errs) {
        $setup->msg($FATAL, @errs);
        return 0;
    }
    $! = 0; # clear errno
    chmod 0770, $rundir;
    if ($!) {
        $setup->msg($FATAL, 'error_chmoding_directory', $rundir, $!);
        return 0;
    }
    chown -1, $gid, $rundir;
    if ($!) {
        $setup->msg($FATAL, 'error_chgrping_directory', $rundir,
                    $setup->{inf}->{General}->{SuiteSpotGroup}, $!);
        return 0;
    }

    return 1;
}

sub makeConfFiles {
    my $setup = shift;
    my $configdir = shift;

    my @start_slapd;
    if ($setup->{inf}->{slapd}->{SlapdConfigForMC} =~ /yes/i) {
        my $inst_dir = $setup->{inf}->{slapd}->{inst_dir};
        @start_slapd = ('ldapStart', "$inst_dir/start-slapd");
    }
    $setup->msg('updating_admconf');
    my $rc = updateAdmConf({ldapurl => $setup->{inf}->{General}->{ConfigDirectoryLdapURL},
                            SuiteSpotUserID => $setup->{inf}->{General}->{SuiteSpotUserID},
                            SuiteSpotGroup => $setup->{inf}->{General}->{SuiteSpotGroup},
                            sysuser => $setup->{inf}->{admin}->{SysUser},
                            sysgroup => $setup->{inf}->{General}->{SuiteSpotGroup},
                            AdminDomain => $setup->{inf}->{General}->{AdminDomain},
                            @start_slapd},
                           $configdir);
    if (!$rc) {
        $setup->msg($FATAL, 'error_updating_admconf', $!);
        return 0;
    }

    $setup->msg('updating_admpw');
    $rc = updateAdmpw($setup->{inf}->{admin}->{ServerAdminID},
                      $setup->{inf}->{admin}->{ServerAdminPwd},
                      $configdir);
    if (!$rc) {
        $setup->msg($FATAL, 'error_updating_admpw');
        return 0;
    }

    return 1;
}

# This is how we extract the sie and isie as the as entries are
# being added
sub registercb {
    my ($context, $entry, $errs) = @_;

    my $rc = check_and_add_entry([$context->{conn}, $context->{reconfig}], $entry, $errs);
    my $setup = $context->{setup};
    if ($rc) {
        if ($entry->hasValue('objectclass', 'nsApplication', 1)) {
            $context->{isie} = $entry->getDN();
        } elsif ($entry->hasValue('objectclass', 'nsAdminServer', 1)) {
            $context->{sie} = $entry->getDN();
        }

        if ($context->{sie}) {
            $rc = updateLocalConf($entry, $context->{sie}, $context->{localfh});
            if (!$rc) {
                $setup->msg($FATAL, 'error_updating_localconf_entry', $entry->getDN());
            }
        }
    } else {
        $setup->msg(@{$errs});
        $setup->msg($FATAL, 'error_adding_adminserver_config_entry', $entry->getDN());
    }

    return $rc;
}

sub registerASWithConfigDS {
    my $setup = shift;
    my $inf = $setup->{inf};
    my $configdir = shift;
    my $reconfig = shift;
    my @errs;

    $setup->msg('registering_adminserver');
    # open a connection to the configuration directory server
    my $conn = getConfigDSConn($inf->{General}->{ConfigDirectoryLdapURL},
                               $inf->{General}->{ConfigDirectoryAdminID},
                               $inf->{General}->{ConfigDirectoryAdminPwd},
                               $configdir, \@errs);

    if (@errs) {
        $setup->msg($FATAL, @errs);
        return 0;
    }

    # add the Admin Server configuration entries
    my @ldiffiles = ("/usr/share/dirsrv/data/20asdata.ldif.tmpl",
                     "/usr/share/dirsrv/data/21astasks.ldif.tmpl",
                     "/usr/share/dirsrv/data/22ascommands.ldif.tmpl"
                     );
    my $setupinf = new Inf("/usr/share/dirsrv/inf/setup.inf");
    my $admininf = new Inf("/usr/share/dirsrv/inf/admin.inf");

    my $mapper = new Inf("/usr/share/dirsrv/inf/adminserver.map");

    $mapper = process_maptbl($mapper, \@errs, $inf, $admininf, $setupinf);
    if (!$mapper or @errs) {
        $conn->close();
        $setup->msg(@errs);
        $setup->msg($FATAL, 'error_creating_adminserver_maptbl');
        return 0;
    }

    # context will get filled in with isie and sie in registercb
    my $localconf = "$configdir/local.conf";
    my $isnew;
    if (! -f $localconf) {
        $isnew = 1;
    }
    if (!open(LOCALCONF, ">$localconf")) {
        $setup->msg($FATAL, 'error_updating_localconf', $localconf, $!);
        return 0;
    }
    my $context = {conn => $conn, localfh => \*LOCALCONF, setup => $setup, reconfig => $reconfig};
    getMappedEntries($mapper, \@ldiffiles, \@errs, \&registercb, $context);
    close(LOCALCONF);

    if ($isnew) {
        my $admConf = getAdmConf($configdir);
        my $uid = getpwnam $admConf->{sysuser};
        chmod 0600, "$localconf";
        chown $uid, -1, "$localconf";        
    }

    $setup->msg('updating_admconf_configds');
    if ($context->{sie} or $context->{isie}) {
        if (!updateAdmConf({sie => $context->{sie},
                            isie => $context->{isie},
                            userdn => $conn->{adminbinddn}},
                           $configdir)) {
            $setup->msg($FATAL, 'error_updating_admconf', $!);
            return 0;
        }
    }

    $conn->close();
    return @errs ? 0 : 1;
}

# update other config files - these are the fields which users typically want to
# change during an install or an upgrade, that also must be synced to the Apache
# style config files - we use the config CGI in command line mode because it
# already has all of the logic to update the files correctly
sub updateHttpConfFiles {
    my $serverAddress = shift;
    my $port = shift;
    my $configdir = shift;
    my $origport = shift;
    my $admConf = getAdmConf($configdir);
    my $user = $admConf->{sysuser};

    # this is required on some platforms in order to execute the config command
    my $savepath = $ENV{SHLIB_PATH} || $ENV{LD_LIBRARY_PATH};
    $ENV{LD_LIBRARY_PATH} = "/usr/lib:$savepath";
    $ENV{SHLIB_PATH} = $ENV{LD_LIBRARY_PATH};
    my $cmd = "/usr/lib/dirsrv/cgi-bin/config op=set configuration.nsSuiteSpotUser=\"$user\"";
    if (!defined($origport) or ($port != $origport)) { # need to change the port number
        $cmd .= " configuration.nsServerPort=\"$port\"";
    }
    if ($serverAddress) {
        $cmd .= " configuration.nsServerAddress=\"$serverAddress\"";
    }

    $? = 0; # clear error
    my $output = `$cmd 2>&1`;
    if ($?) {
        debug(0, $output);
        $ENV{LD_LIBRARY_PATH} = $savepath;
        $ENV{SHLIB_PATH} = $savepath;
        return 0;
    }

    debug(1, $output);
    $ENV{LD_LIBRARY_PATH} = $savepath;
    $ENV{SHLIB_PATH} = $savepath;

    # update Group in console.conf
    if ($admConf->{sysgroup}) {
        if (!open(CONSOLECONF, "$admConf->{configdir}/console.conf")) {
            debug(0, "Error opening $admConf->{configdir}/console.conf: $!");
            return 0;
        }
        my @contents = <CONSOLECONF>;
        close (CONSOLECONF);
        grep { s/^Group.*$/Group $admConf->{sysgroup}/ } @contents;
        if (!open(CONSOLECONF, ">$admConf->{configdir}/console.conf")) {
            debug(0, "Error writing new group $admConf->{sysgroup} to $admConf->{configdir}/console.conf: $!");
            return 0;
        }
        print CONSOLECONF @contents;
        close (CONSOLECONF);
    }
    return 1;
}

sub startAdminServer {
    my $setup = shift;
    my $configdir = shift;
    my $logdir = shift;
    my $rundir = shift;
    my $isrunning;

    $pidfile = "$rundir/admin-serv.pid";
    if (-f $pidfile) {
        open(PIDFILE, $pidfile);
        my $pid = <PIDFILE>;
        close(PIDFILE);
        if (kill 0, $pid) {
            $isrunning = 1;
        }
    }

    my ($fh, $filename) = tempfile("asstartupXXXXXX", UNLINK => 1,
                                   SUFFIX => ".log", DIR => File::Spec->tmpdir);
    close($fh);
    my $rc;
    if ($isrunning) {
        $setup->msg('restarting_adminserver');
        $rc = system("/usr/sbin/restart-ds-admin > $filename 2>&1");
    } else {
        $setup->msg('starting_adminserver');
        $rc = system("/usr/sbin/start-ds-admin > $filename 2>&1");
    }

    open(STARTLOG, "$filename");
    while (<STARTLOG>) {
        $setup->msg('adminserver_startup_output', $_);
    }
    close(STARTLOG);
    unlink($filename);

    if ($rc) {
        $setup->msg($FATAL, 'error_starting_adminserver', $rc);
        return 0;
    }

    $setup->msg('success_starting_adminserver');
    return 1;
}

sub createAdminServer {
    my $setup = shift;
    my $reconfig = shift;
    # setup has inf, res, and log

    if ($reconfig) {
        $setup->msg('begin_reconfig_adminserver');
    } else {
        $setup->msg('begin_create_adminserver');
    }

    if (!checkRequiredParameters($setup)) {
        return 0;
    }

    my $configdir = $setup->{inf}->{admin}->{config_dir} ||
        $ENV{ADMSERV_CONF_DIR} ||
        $setup->{configdir} . "/admin-serv";

    my $securitydir = $setup->{inf}->{admin}->{security_dir} ||
        $configdir;

    my $logdir = $setup->{inf}->{admin}->{log_dir} ||
        $ENV{ADMSERV_LOG_DIR} ||
        "/var/log/dirsrv/admin-serv";

    my $rundir = $setup->{inf}->{admin}->{run_dir} ||
        $ENV{ADMSERV_PID_DIR} ||
        "/var/run/dirsrv";

    if (!createASFilesAndDirs($setup, $configdir, $securitydir, $logdir, $rundir)) {
        return 0;
    }

    if (!makeConfFiles($setup, $configdir)) {
        return 0;
    }

    if (!registerASWithConfigDS($setup, $configdir, $reconfig)) {
        return 0;
    }

    $setup->msg('updating_httpconf');
    if (!updateHttpConfFiles($setup->{inf}->{admin}->{ServerIpAddress},
                             $setup->{inf}->{admin}->{Port},
                             $configdir, $setup->{asorigport})) {
        $setup->msg($FATAL, 'error_updating_httpconf');
        return 0;
    }

    if (!setFileOwnerPerms($setup, $configdir)) {
        return 0;
    }

    if (!startAdminServer($setup, $configdir, $logdir, $rundir)) {
        return 0;
    }

    if ($reconfig) {
        $setup->msg('end_reconfig_adminserver');
    } else {
        $setup->msg('end_create_adminserver');
    }
    return 1;
}

sub reconfigAdminServer {
    my $setup = shift;
    return createAdminServer($setup, 1);
}

1;

# emacs settings
# Local Variables:
# mode:perl
# indent-tabs-mode: nil
# tab-width: 4
# End:
